/**
 * P4P Vuze Plugin
 * 
 * Copyright 2008, Yale Laboratory of Networked Systems
 */

package net.yale.cs.p4p.p2p.azureus.client;

import org.gudy.azureus2.plugins.ui.model.BasicPluginConfigModel;

public final class Config
{
	private static final String RESOURCE_NAME_PREFIX = "net.yale.cs.p4p.";

	private static final String RESOURCE_NAMES_PATH = "net.yale.cs.p4p.p2p.azureus.client.Messages";
	private static final String RESOURCE_NAME_DISPLAY = RESOURCE_NAME_PREFIX + "PluginName";

	private class ConfigItem<T> {
		private String resourceName_;
		public final String getResourceName() { return RESOURCE_NAME_PREFIX + resourceName_; }

		private String key_;
		public final String getKey() { return key_; }
		
		private T default_;
		public final T getDefault() { return default_; }

		public ConfigItem(String resourceName, T dflt) {
			resourceName_ = resourceName;
			key_ = resourceName.toUpperCase();
			default_ = dflt;
		}
	}

	private final class ConfigItemString extends ConfigItem<String> {
		public ConfigItemString(String resourceName, String dflt) {
			super(resourceName, dflt);
		}
		public String getValue() {
			return plugin_.getPluginInterface().getPluginconfig().getPluginStringParameter(getKey(), getDefault());
		}
		public void addToConfig(BasicPluginConfigModel cfgView) {
			cfgView.addStringParameter2(getKey(), getResourceName(), getDefault());
		}
	}

	private final class ConfigItemInteger extends ConfigItem<Integer> {
		public ConfigItemInteger(String resourceName, Integer dflt) {
			super(resourceName, dflt);
		}
		public Integer getValue() {
			return plugin_.getPluginInterface().getPluginconfig().getPluginIntParameter(getKey(), getDefault());
		}
		public void addToConfig(BasicPluginConfigModel cfgView) {
			cfgView.addIntParameter2(getKey(), getResourceName(), getDefault());
		}
	}

	private final class ConfigItemFloat extends ConfigItem<Float> {
		public ConfigItemFloat(String resourceName, Float dflt) {
			super(resourceName, dflt);
		}
		public Float getValue() {
			return plugin_.getPluginInterface().getPluginconfig().getPluginFloatParameter(getKey(), getDefault());
		}
		public void addToConfig(BasicPluginConfigModel cfgView) {
			/* FIXME: Adding floats is currently not provided in interface */
		}
	}

	private final class ConfigItemBoolean extends ConfigItem<Boolean> {
		public ConfigItemBoolean(String resourceName, Boolean dflt) {
			super(resourceName, dflt);
		}
		public Boolean getValue() {
			return plugin_.getPluginInterface().getPluginconfig().getPluginBooleanParameter(getKey(), getDefault());
		}
		public void addToConfig(BasicPluginConfigModel cfgView) {
			cfgView.addBooleanParameter2(getKey(), getResourceName(), getDefault());
		}
	}

	private ConfigItemBoolean cfgEnabled_ = new ConfigItemBoolean("Enabled", true);
	public boolean getEnabled() { return cfgEnabled_.getValue(); }

	private ConfigItemInteger cfgPIDQueryInterval_ = new ConfigItemInteger("PIDQueryInterval", 1);
	public int getPIDQueryInterval() { return cfgPIDQueryInterval_.getValue(); }

	private ConfigItemInteger cfgPIDQueryBatch_ = new ConfigItemInteger("PIDQueryBatch", 50);
	public int getPIDQueryBatch() { return cfgPIDQueryBatch_.getValue(); }

	private ConfigItemInteger cfgPIDQuerySelfInterval_ = new ConfigItemInteger("PIDQuerySelfInterval", 24 * 60 * 60);
	public int getPIDQuerySelfInterval() { return cfgPIDQuerySelfInterval_.getValue(); }

	private ConfigItemInteger cfgPIDMapQueryInterval_ = new ConfigItemInteger("PIDMapQueryInterval", 3 * 24 * 60 * 60);
	public int getPIDMapQueryInterval() { return cfgPIDMapQueryInterval_.getValue(); }

	private ConfigItemInteger cfgPDistQueryInterval_ = new ConfigItemInteger("PDistQueryInterval", 20 * 60);
	public int getPDistQueryInterval() { return cfgPDistQueryInterval_.getValue(); }

	private ConfigItemInteger cfgPortalDiscoverInterval_ = new ConfigItemInteger("PortalDiscoverInterval", 20 * 60);
	public int getPortalDiscoverInterval() { return cfgPortalDiscoverInterval_.getValue(); }

	private ConfigItemString cfgPortalHost_ = new ConfigItemString("PortalHost", "");
	public String getPortalHost() { return cfgPortalHost_.getValue(); }

	private ConfigItemInteger cfgPortalPort_ = new ConfigItemInteger("PortalPort", 6671);
	public int getPortalPort() { return cfgPortalPort_.getValue(); }

	private ConfigItemString cfgPortalDBHost_ = new ConfigItemString("PortalDBHost", "p4p-6.cs.yale.edu");
	public String getPortalDBHost() { return cfgPortalDBHost_.getValue(); }

	private ConfigItemInteger cfgPortalDBPort_ = new ConfigItemInteger("PortalDBPort", 6670);
	public int getPortalDBPort() { return cfgPortalDBPort_.getValue(); }

	private ConfigItemInteger cfgMinDownloadBPS_ = new ConfigItemInteger("MinDownloadBPS", 1 * 1024 * 1024);
	public int getMinDownloadBPS() { return cfgMinDownloadBPS_.getValue(); }

	private ConfigItemInteger cfgMaxDownloadBPS_ = new ConfigItemInteger("MaxDownloadBPS", 200 * 1024 * 1024);
	public int getMaxDownloadBPS() { return cfgMaxDownloadBPS_.getValue(); }

	private ConfigItemInteger cfgPeerLifetimeMultiplier_ = new ConfigItemInteger("PeerLifetimeMultiplier", 2);
	public int getPeerLifetimeMultiplier() { return cfgPeerLifetimeMultiplier_.getValue(); }

	private ConfigItemInteger cfgMaxSelectedPeers_ = new ConfigItemInteger("MaxSelectedPeers", 100);
	public int getMaxSelectedPeers() { return cfgMaxSelectedPeers_.getValue(); }

	private ConfigItemFloat cfgSimpleAlgIntrapidPct_ = new ConfigItemFloat("SimpleAlgIntrapidPct", 0.70f);
	public float getSimpleAlgIntrapidPct() { return cfgSimpleAlgIntrapidPct_.getValue(); }

	private ConfigItemFloat cfgSimpleAlgIntradomainPct_ = new ConfigItemFloat("SimpleAlgIntradomainPct", 0.80f);
	public float getSimpleAlgIntradomainPct() { return cfgSimpleAlgIntradomainPct_.getValue(); }

	private ConfigItemFloat cfgSimpleAlgInterdomainPct_ = new ConfigItemFloat("SimpleAlgInterdomainPct", 1.00f);
	public double getSimpleAlgInterdomainPct() { return cfgSimpleAlgInterdomainPct_.getValue(); }

	private ConfigItemFloat cfgSimpleAlgIntrapidUnchokePct_ = new ConfigItemFloat("SimpleAlgIntrapidUnchokePct", 4.0f / 7.0f);
	public float getSimpleAlgIntrapidUnchokePct() { return cfgSimpleAlgIntrapidUnchokePct_.getValue(); }

	private ConfigItemFloat cfgSimpleAlgIntradomainUnchokePct_ = new ConfigItemFloat("SimpleAlgIntradomainUnchokePct", 6.0f / 7.0f);
	public float getSimpleAlgIntradomainUnchokePct() { return cfgSimpleAlgIntradomainUnchokePct_.getValue(); }

	private ConfigItemFloat cfgSimpleAlgInterdomainUnchokePct_ = new ConfigItemFloat("SimpleAlgInterdomainUnchokePct", 1.00f);
	public double getSimpleAlgInterdomainUnchokePct() { return cfgSimpleAlgInterdomainUnchokePct_.getValue(); }

	private ConfigItemInteger cfgPEXCheckIntervalMs_ = new ConfigItemInteger("PEXCheckIntervalMS", 2 * 1000);
	public int getPEXCheckIntervalMS() { return cfgPEXCheckIntervalMs_.getValue(); }

	private ConfigItemInteger cfgPeerScheduleDelay_ = new ConfigItemInteger("PeerScheduleDelay", 5);
	public int getPeerScheduleDelay() { return cfgPeerScheduleDelay_.getValue(); }

	private ConfigItemInteger cfgExperimentQueryInterval_ = new ConfigItemInteger("ExperimentQueryInterval", 2 * 60 * 60);
	public int getExperimentQueryInterval() { return cfgExperimentQueryInterval_.getValue(); }

	private ConfigItemBoolean cfgExperimentOverrideEnabled_ = new ConfigItemBoolean("ExperimentOverrideEnabled", false);
	public boolean getExperimentOverrideEnabled() { return cfgExperimentOverrideEnabled_.getValue(); }

	private ConfigItemString cfgExperimentHost_ = new ConfigItemString("ExperimentHost", "p4p-2.cs.yale.edu");
	public String getExperimentHost() { return cfgExperimentHost_.getValue(); }

	private ConfigItemInteger cfgExperimentPort_ = new ConfigItemInteger("ExperimentPort", 6675);
	public int getExperimentPort() { return cfgExperimentPort_.getValue(); }

	private P4PPlugin plugin_;
	private BasicPluginConfigModel cfgView_;

	public Config(P4PPlugin plugin) {
		plugin_ = plugin;
		
		/* Install our resource names */
		plugin_.getPluginInterface().getUtilities().getLocaleUtilities().integrateLocalisedMessageBundle(RESOURCE_NAMES_PATH);

		/* Setup our config parameters in the UI */
		cfgView_ = plugin_.getPluginInterface().getUIManager().createBasicPluginConfigModel(RESOURCE_NAME_DISPLAY);
		cfgEnabled_.addToConfig(cfgView_);
		if (P4PPlugin.IS_DEBUG) {
			cfgPIDQueryInterval_.addToConfig(cfgView_);
			cfgPIDQueryBatch_.addToConfig(cfgView_);
			cfgPIDQuerySelfInterval_.addToConfig(cfgView_);
			cfgPIDMapQueryInterval_.addToConfig(cfgView_);
			cfgPDistQueryInterval_.addToConfig(cfgView_);
			cfgPortalDiscoverInterval_.addToConfig(cfgView_);
			cfgPortalHost_.addToConfig(cfgView_);
			cfgPortalPort_.addToConfig(cfgView_);
			cfgPortalDBHost_.addToConfig(cfgView_);
			cfgPortalDBPort_.addToConfig(cfgView_);
			cfgMinDownloadBPS_.addToConfig(cfgView_);
			cfgMaxDownloadBPS_.addToConfig(cfgView_);
			cfgPeerLifetimeMultiplier_.addToConfig(cfgView_);
			cfgMaxSelectedPeers_.addToConfig(cfgView_);
			cfgSimpleAlgIntrapidPct_.addToConfig(cfgView_);
			cfgSimpleAlgIntradomainPct_.addToConfig(cfgView_);
			cfgSimpleAlgInterdomainPct_.addToConfig(cfgView_);
			cfgPEXCheckIntervalMs_.addToConfig(cfgView_);
			cfgPeerScheduleDelay_.addToConfig(cfgView_);
			cfgExperimentQueryInterval_.addToConfig(cfgView_);
			cfgExperimentOverrideEnabled_.addToConfig(cfgView_);
		}
	}

	public void cleanup() {
		cfgView_.destroy();
	}
}
