/**
 * P4P Vuze Plugin
 * 
 * Copyright 2008, Yale Laboratory of Networked Systems
 */

package net.yale.cs.p4p.p2p.azureus.client.data;

import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;

import net.yale.cs.p4p.api.APIBase.HTTPErrorException;
import net.yale.cs.p4p.api.data.InetService;
import net.yale.cs.p4p.api.data.PID;
import net.yale.cs.p4p.api.data.PIDDestVector;
import net.yale.cs.p4p.api.data.PIDMatrix;
import net.yale.cs.p4p.p2p.azureus.client.P4PPlugin;
import net.yale.cs.p4p.portal.api.QueryREST;

public final class PDistCache extends QueryCache
{
	private static final Collection<PID> EMPTY_PID_LIST = new LinkedList<PID>();

	private final class PDistQueryTask extends QueryTask {

		public PDistQueryTask(int interval) {
			super(interval);
		}

		protected void query() throws InterruptedException {
			/* Ignore if no portal server available */
			InetService portalServer = peerCache_.getPortalServer();
			if (portalServer == null)
				return;

			/* Ignore if we don't have a PID */
			PID selfPid = peerCache_.getSelfPID();
			if (selfPid == null)
				return;

			/* Query and replace the existing set of pDistances */
			getPlugin().getLogger().log("Querying updated pDistances");
			try {
				pdists_ = new QueryREST(portalServer).getVCost(Collections.singletonList(new PIDDestVector(selfPid, EMPTY_PID_LIST, true)));
				getPlugin().getLogger().log("Finished querying for updated pDistances");
			} catch (HTTPErrorException e) {
				getPlugin().getLogger().log("Portal discovery failed: HTTP error " + e.getCode() + " " + e.getReason());
			} catch (IOException e) {
				getPlugin().getLogger().log("Failed to update pDistances from Portal Server: " + e);
			}
			
			setInterval(getPlugin().getConfig().getPDistQueryInterval() * 1000);
		}
	}

	private PeerCache peerCache_;

	private PIDMatrix pdists_ = new PIDMatrix();
	public PIDMatrix getPDists() { return pdists_; }

	public PDistCache(P4PPlugin plugin, int queryInterval, PeerCache peerCache) {
		super(plugin, 1);
		peerCache_ = peerCache;
	}

	protected QueryTask makeQueryTask() {
		return new PDistQueryTask(getPlugin().getConfig().getPDistQueryInterval() * 1000);
	}

}
