package net.yale.cs.p4p.p2p.azureus.client.experiment;

import java.io.IOException;
import java.net.URL;

import net.yale.cs.p4p.api.data.PID;
import net.yale.cs.p4p.p2p.azureus.client.P4PPlugin;
import net.yale.cs.p4p.p2p.azureus.client.data.QueryCache;

import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.torrent.Torrent;
import org.gudy.azureus2.plugins.torrent.TorrentAnnounceURLListSet;

public final class ExperimentDecider extends QueryCache {

	private class ExperimentQuery extends QueryTask {

		public ExperimentQuery(int interval) {
			super(interval);
		}

		protected void query() throws InterruptedException {
			try {
				getPlugin().getLogger().log("Requesting experiment configuration");
				params_ = getPlugin().getExperimentInterface().getExperimentParams();
				getPlugin().getLogger().log("Finished equesting experiment configuration");
			} catch (IOException e) {
				getPlugin().getLogger().log("Failed to request experiment configuration: " + e);
			}
			
			setInterval(getPlugin().getConfig().getExperimentQueryInterval() * 1000);
		}

	}

	private ExperimentParams params_ = new ExperimentParams();
	public ExperimentParams getParams() { return params_; }

	public ExperimentDecider(P4PPlugin plugin) {
		super(plugin, 1);
	}

	protected QueryTask makeQueryTask() {
		return new ExperimentQuery(getPlugin().getConfig().getExperimentQueryInterval() * 1000);
	}
	
	private boolean isISPEnabled(ExperimentParams params) {
		PID selfPid = null;
		
		switch (params.getISPMode()) {
		case ISP_ENABLED:
			return true;
		case ISP_DISABLED:
			return false;
		case ISP_BY_ISPID:
			selfPid = getPlugin().getPeerCache().getSelfPID();
			if (selfPid == null)
				return false;
			return params.isISPEnabled(selfPid.getISP());
		default:
			return true;
		}
	}
	
	private boolean isClientEnabled(ExperimentParams params) {
		byte[] clientId = getPlugin().getPluginInterface().getUtilities().getSecurityManager().getIdentity();
		switch (params.getClientMode()) {
		case CLIENT_ENABLED:
			return true;
		case CLIENT_DISABLED:
			return false;
		case CLIENT_BY_CLIENTID_BIT:
			return clientId == null || (clientId[0] & 0x1) != 0;
		default:
			return true;
		}
	}

	private boolean isTorrentEnabled(ExperimentParams params, Torrent torrent) {
		switch (params.getTorrentMode()) {
		case TORRENT_ENABLED:
			return true;
		case TORRENT_DISABLED:
			return false;
		case TORRENT_BY_HASH_BIT:
			return (torrent.getHash()[0] & 0x1) != 0;
		case TORRENT_BY_HASH:
			return params.isTorrentHashEnabled(torrent.getHash());
		case TORRENT_BY_TRACKER:
			if (params.isTrackerHostEnabled(torrent.getAnnounceURL().getHost()))
				return true;
			for (TorrentAnnounceURLListSet announceList : torrent.getAnnounceURLList().getSets())
				for (URL announceUrl : announceList.getURLs())
					if (params.isTrackerHostEnabled(announceUrl.getHost()))
						return true;
			return false;
		default:
			return true;
		}
	}

	public boolean isP4PEnabled(Download download) {
		ExperimentParams params = getParams();

		/* Disable if no torrent */
		Torrent torrent = download.getTorrent();
		if (torrent == null)
			return false;
		
		/* Disable if we're not actively downloading (e.g., we're seeding) */
		if (download.getState() != Download.ST_DOWNLOADING)
			return false;

		/* Enable if explicitly configured */
		if (getPlugin().getConfig().getExperimentOverrideEnabled())
			return true;

		/* First check the ISP */
		if (!isISPEnabled(params))
			return false;
		
		/* Next check the Client */
		if (!isClientEnabled(params))
			return false;
		
		/* Finally check the torrent */
		if (!isTorrentEnabled(params, torrent))
			return false;
		
		/* Enable if all checks pass */
		return true;
	}

}
