/**
 * P4P Vuze Plugin
 * 
 * Copyright 2008, Yale Laboratory of Networked Systems
 */

package net.yale.cs.p4p.p2p.azureus.client.listeners;

import java.util.HashMap;
import java.util.Map;

import net.yale.cs.p4p.p2p.azureus.client.P4PPlugin;

import org.gudy.azureus2.plugins.download.Download;
import org.gudy.azureus2.plugins.download.DownloadPeerListener;
import org.gudy.azureus2.plugins.peers.PeerManager;
import org.gudy.azureus2.plugins.torrent.Torrent;

public class P4PDownloadPeerListener implements DownloadPeerListener {
	
	private P4PPlugin plugin_;

	private Map<PeerManager, P4PPeerManagerListener> listeners_ = new HashMap<PeerManager, P4PPeerManagerListener>();

	public P4PDownloadPeerListener(P4PPlugin plugin) {
		plugin_ = plugin;
	}

	public void peerManagerAdded(Download download, PeerManager peer_manager) {
		if (!listeners_.containsKey(peer_manager)) {
			P4PPeerManagerListener l = new P4PPeerManagerListener(plugin_, peer_manager);
			listeners_.put(peer_manager, l);
			peer_manager.addListener(l);
		}
	}

	public void peerManagerRemoved(Download download, PeerManager peer_manager) {
		Torrent torrent = download.getTorrent();
		if (torrent != null)
			plugin_.getTorrents().removeTorrent(torrent);

		P4PPeerManagerListener l = listeners_.remove(peer_manager);
		if (l != null) {
			l.cleanup();
			peer_manager.removeListener(l);
		}
	}

	public void cleanup() {
		for (Map.Entry<PeerManager, P4PPeerManagerListener> e : listeners_.entrySet()) {
			e.getValue().cleanup();
			e.getKey().removeListener(e.getValue());
		}
	}

}
